{*
Data unit
some data types for information handling

@author         Philipp Hartl
@version        0.1
@license
This source file is part of BotanicalVis,
a tool to visualize huge hierarchies in 3D-space

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
*}

unit Data;

interface

uses
  Classes, SysUtils, ComCtrls;

type

  /// a special list for pdi (PTDirectoryItem) structure
  TSList = class(TList)
  private
    FNextIndex: integer;
    function GetCount: integer;

  public
    constructor Create;
    function Next: Pointer;     /// return next element and increment FNextIndex
    function Pop: Pointer;      /// pop next element
    function PopFirst: Pointer; /// pop first element and shrink list
    procedure Rewind;

    property PCount: integer read GetCount;
  end;

  TItem = (FILE_ITEM, DIR_ITEM);

  /// structure for information of a file
  TFileItem = record
    n: String;          /// full name of file (with extension)
    s: Int64;           /// size in byte
  end;
  PTFileItem = ^TFileItem;

  /// a structure for a directory listing
  TDirectoryItem = record
    n: String;          /// name of directory
    s: Int64;           /// directory size (strand size)
    fileCnt: Cardinal;  /// sum of files (incl subdirs)
    dirCnt: Cardinal;   /// sum of dirs
    tn: TTreeNode;      /// pointer on a treenode
    dl: TSList;         /// list of PDirectoryItem(s);
    fl: TSList;         /// list of FileItem(s);
  end;
  PTDirectoryItem = ^TDirectoryItem;

  /// return size in user friendly size
  function PrintSizeUserFriendly(const size: Int64): String;

implementation

//------------------------------------------------------------------------------

{*
format the given size to user friendly dimensions (KB, MB, GB)

@param  size, the value we want to format
*}
function PrintSizeUserFriendly(const size: Int64): String;
const dim: array[0..2] of String = ('Kb','Mb','Gb');
var
  tmp: Int64;
  i: shortint;
begin
  result:= IntToStr(size) + 'b';
  if size < 1024 then exit;

  i:= -1;
  tmp:= size;
  while tmp >= 1024 do
  begin
    tmp:= tmp shr 10;
    INC(i);
  end;
  result:= IntToStr(tmp) + dim[i] + ' (' + result + ')';
end;

//------------------------------------------------------------------------------
// TSList
//------------------------------------------------------------------------------

constructor TSList.Create;
begin
  inherited Create;
  FNextIndex:= 0;
end;

{*
how many items are left, after pop
*}
function TSList.GetCount: integer;
begin
  result:= Count - FNextIndex;
end;

{*
return the next element from the list, without extracting it

@return the next pointer in list, otherwise nil
*}
function TSList.Next: Pointer;
begin
  result:= nil;
  if FNextIndex < Count then
  begin
    result:= Get(FNextIndex);
    Inc(FNextIndex);
  end;
end;

{*
pop next element from the list
remove it from the list
*}
function TSList.Pop: Pointer;
begin
  result:= Extract(Next);
end;

{*
pop first element and shrink the list
*}
function TSList.PopFirst: Pointer;
begin
  result:= Extract(First);
end;

{*
to use pop again call rewind
*}
procedure TSList.Rewind;
begin
  FNextIndex:= 0;
end;

end.