{*
Logger unit
simple file logging usage
singleton implementation

@author         Philipp Hartl
@version        1.0
@license
This source file is part of BotanicalVis,
a tool to visualize huge hierarchies in 3D-space

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
*}
unit Logger;

interface

uses
  Classes, SysUtils;

type

  TLogger = class(TFileStream)
  protected
    constructor Create;                         /// should not be called directly
  public
    destructor Destroy; override;
    class function GetInstance: TLogger;        /// returns the singleton instance of TLogger
    procedure LogMessage(const msg: String);    /// logging function
  end;

var
  LogInstance: TLogger = nil;                  /// the singleton instance object of TLogger, do not use directly

implementation

//------------------------------------------------------------------------------
// TLogger
//------------------------------------------------------------------------------

{*
create a file with application name and .log extension
and open it for exlusive writting, nothing should have access until freeing
*}
constructor TLogger.create;
var
  s: String;
begin
  s:= ParamStr(0);
  s:= copy(s, 0, Length(s) - 4) + '.log';
  LogInstance:= inherited create(s, fmCreate or fmShareExclusive);
end;

{*
standard destuctor
*}
destructor TLogger.destroy;
begin
  LogInstance:= nil;
  inherited destroy;
end;

{*
class function to access the singleton instance of TLogger

@return singleton TLogger instance
*}
class function TLogger.GetInstance: TLogger;
begin
  if not Assigned(LogInstance) then Create;
  result:= LogInstance;
end;

{*
write msg to file
*}
procedure TLogger.LogMessage(const msg: String);
var
  s: String;
begin
  // add timestamp
  s:= TimeToStr(Time) + ': ' + msg + #13#10;
  Write(s[1], Length(s));
end;

end.
